@@ -1,5 +1,5 @@
-import os
-import PySide2
+# import os
+# import PySide2
 
 # Resolve PySide2 plugin issue, should be imported prior to using of PySide2.
 # Entire errror message was like the followings:
@@ -12,6 +12,6 @@ import PySide2
 # I think this is happening due to anancodna environment,
 # since Qt dlls are added to PATH under an ananconda environment.
 #
-if hasattr(PySide2, '__file__'):
-    qt_qpa_platform_plugin_path = os.path.join(os.path.dirname(PySide2.__file__), 'plugins', 'platforms')
-    os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = qt_qpa_platform_plugin_path
+# if hasattr(PySide2, '__file__'):
+#     qt_qpa_platform_plugin_path = os.path.join(os.path.dirname(PySide2.__file__), 'plugins', 'platforms')
+#     os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = qt_qpa_platform_plugin_path
